
# A DlistMembership extends the regular member adaptor to back certain
# attributes with a database.
# Additionally, this implements support for loose matching.


from OldStyleMemberships import OldStyleMemberships as oldm
from Mailman import DlistUtils
from Mailman import mm_cfg
from Mailman.Logging.Syslog import syslog
from Mailman import MemberAdaptor
#from mm_cfg import DEBUG_MODE

ISREGULAR = 1
ISDIGEST = 2


class DlistMemberships(oldm):

    def __init__(self, mlist):
        oldm.__init__(self, mlist)
        self.__mlist = mlist

    def __get_cp_member(self, member):
        # First try strict matching
        result, type = self.__get_cp_member_strict(member)
        if result:
            return result, type
        # Now try loose email matching if this list permits it
        try:
            if self.__mlist.loose_email_matching:
                return self.__get_cp_member_loose(member)
        except:
            pass
        return None, None

    # Name changed from __get_cp_member
    def __get_cp_member_strict(self, member):
        """Does the member name exactly match the subscribed name?"""
        lcmember = member.lower()
        missing = []
        val = self.__mlist.members.get(lcmember, missing)
        if val is not missing:
            if isinstance(val, StringType):
                return val, ISREGULAR
            else:
                return lcmember, ISREGULAR
        val = self.__mlist.digest_members.get(lcmember, missing)
        if val is not missing:
            if isinstance(val, StringType):
                return val, ISDIGEST
            else:
                return lcmember, ISDIGEST
        return None, None

    def __get_cp_member_loose(self, member):
        """Check if the provided member email address loosely matches any list membership.
        This should only be called if the list permits loose matching.
        We could save time in the future by adding matches to the alias database."""
        lcmember = member.lower()
        try:
            at_split = lcmember.split('@')
            if len(at_split) != 2:
                return None, None
            local = at_split[0]
            domain = at_split[1]

            # Check if member argument is narrower than actual membership;
            # e.g., robin@eng.sun.com vs. robin@sun.com
            domain_split = domain.split('.')[1:]
            while len(domain_split) > 1:
                substring = local + '@' + string.join(domain_split, '.')
                result, type = self.__get_cp_member_strict(substring)
                if result:
                    return result, type
                domain_split = domain_split[1:]

            # e.g., robin@sun.com vs. robin@eng.sun.com
            regstring = local + '@.*\.' + domain
            try:
                regexp = re.compile(regstring)
            except:
                syslog('error', 'OldStyleMemberships.__get_cp_member_loose: error in re.compile')
            for mem in self.__mlist.members.keys():
                if regexp.match(mem):
                    val = self.__mlist.members.get(mem)
                    if isinstance(val, StringType):
                        return val, ISREGULAR
                    else:
                        return mem, ISREGULAR
            for mem in self.__mlist.digest_members.keys():
                if regexp.match(mem):
                    val = self.__mlist.members.get(mem)
                    if isinstance(val, StringType):
                        return val, ISREGULAR
                    else:
                        return mem, ISREGULAR
        # Email address in bad format
        except Error, e:
            #if :
            syslog('error', 'OldStyleMemberships.__get_cp_member_loose: Error in OldStyleMemberships.__get_cp_member_loose')
            syslog('error', e)
            pass

        return None, None

    # override to strip whitespace off passwords
    def authenticateMember(self, member, response):
        secret = self.getMemberPassword(member).strip()
        if secret == response.strip():
            return secret
        return 0

    def addNewMember(self, member, **kws):
        if kws.has_key("affect_dlist_database"):
            affect_dlist_database = kws["affect_dlist_database"]
            del kws["affect_dlist_database"]
        else:
            affect_dlist_database = True
        if kws.has_key("digest"):
            digest = kws["digest"]
        else:
            digest = False
        oldm.addNewMember(self, member, **kws)
        if affect_dlist_database:
	    subscriber = DlistUtils.Subscriber(self.__mlist)
            if DlistUtils.enabled(self.__mlist):
                subscriber.subscribeToList(member)
                if digest:
                    subscriber.setDigest(member, 1)

    def removeMember(self, member, affect_dlist_database=1):
	subscriber = DlistUtils.Subscriber(self.__mlist)
        oldm.removeMember(self, member)
        memberkey = member.lower()
        if affect_dlist_database and DlistUtils.enabled(self.__mlist):
            subscriber.unsubscribeFromList(memberkey)

    def changeMemberAddress(self, member, newaddress, nodelete=1):
	subscriber = DlistUtils.Subscriber(self.__mlist)
        memberkey = member.lower()
        if DlistUtils.enabled(self.__mlist):
            subscriber.changeAddress(memberkey, newaddress)
        oldm.changeMemberAddress(self, member, newaddress, nodelete=0)  #changed nodelete to 0

    def setDeliveryStatus(self, member, status):
	subscriber = DlistUtils.Subscriber(self.__mlist)
        oldm.setDeliveryStatus(self, member, status)
        memberkey = member.lower()
        enabled = (status == MemberAdaptor.ENABLED)
        if DlistUtils.enabled(self.__mlist):
            subscriber.setDisable(member, (not enabled))

    def setMemberOption(self, member, flag, value):
	subscriber = DlistUtils.Subscriber(self.__mlist)
        oldm.setMemberOption(self, member, flag, value)
        if flag == mm_cfg.Digests and DlistUtils.enabled(self.__mlist):
            subscriber.setDigest(member, value)

